function antbias(varargin)
% antbias.m :
% analize glonass interchannel bias from estimated amibiguity by antest.
%
% options : ([]:default)
%    file             out file by antest.m [antprm.mat]
%    '-ref,fn         reference frequency number to generate DD [0]
%    '-unit',unit     unit of bias {'cm'|'cycle'} [cm]
%    '-minep',nep     min number of epochs in a arc to compute bias [30]
%    '-amb'           plot ambiguity parameters instead of interchannel bias
%
%            Copyright(c) 2011 by T.Takasu, all rights reserved
%
% version: $Revision: 1.3 $ $Date: 2008/07/21 06:45:05 $
% history: 2011/12/21 0.1 new

C=299792458;
ref=0;     % freq no of reference satellite
unit='cm'; % unit
ptype=0;   % plot type (0:inter-ch-bias,1:ambiguity)
minep=30;  % min number of epochs
file='antprm.mat';

i=1; fobs={};
while i<=nargin
    switch varargin{i}
    case '-ref',  i=i+1; ref=tonum(varargin{i});
    case '-unit', i=i+1; unit=varargin{i};
    case '-minep',i=i+1; minep=tonum(varargin{i});
    case '-amb',  ptype=1;
    otherwise, file=varargin{i};
    end
    i=i+1;
end

% load estimation results by antest.m
load(file);

% extract glonass data
time=amb(:,1);
amb =amb(:,1+(33:56));
fn  =fn  (:,33:56);
freq=freq(:,33:56);

% m -> cycle
for i=1:size(amb,2)
    amb(:,i)=amb(:,i)/C*freq(opt.f,i);
end

% generate double difference in cycle
for i=1:size(amb,1)
    j=min(find(~isnan(amb(i,:))&fn==ref));
    if ~isempty(j), amb(i,:)=amb(i,:)-amb(i,j); else amb(i,:)=nan; end
end

% plot ambiguity
if ptype
    plotamb(time,amb,opt.f,minep);
    return
end
% extract ambiguty
ambf=extamb(time,amb,fn,ref,minep);

% cycle -> cm
if strcmp(unit,'cm')
    for i=1:size(ambf,1)
        ambf(i,3)=ambf(i,3)*C/freq(opt.f,ambf(i,1))*100;
    end
end
% plot inter channel bias
plotbias(ambf,opt.f,unit);

% extract estimated ambiguity --------------------------------------------------
function ambf=extamb(time,amb,fn,ref,minep)
ambf=[];
for i=1:size(amb,2)
    j=find(~isnan(amb(:,i)));
    
    ti=time(j);
    ambi=amb(j,i);
    
    % detect phase bias break
    k=find(abs(diff(ti))>300|abs(diff(ambi))>0.5);
    for l=[1,k'+1;k',length(ambi)]
        
        if l(2)-l(1)<minep, continue; end
        
        % [sat,fn,ambiguty,n;...]
        ambf=[ambf;i,fn(i),ambi(l(2)),l(2)-l(1)+1];
    end
end
% resolve integer ambiguity
ambf=resamb(ambf,ref);

% resolve integer ambiguity ----------------------------------------------------
function ambf=resamb(ambf,ref)

i=find(ambf(:,2)==ref+1);
j=find(ambf(:,2)==ref-1);
ambf(i,3)=ambf(i,3)-round(ambf(i,3));
ambf(j,3)=ambf(j,3)-round(ambf(j,3));

for n=1:16
    i=find(abs(ambf(:,2)-ref)<=n);
    p=polyfit(ambf(i,2)-ref,ambf(i,3),1);
    for i=find(ambf(:,2)==ref+n+1)
        a=polyval(p,n+1);
        ambf(i,3)=ambf(i,3)-round(ambf(i,3)-a);
    end
    for i=find(ambf(:,2)==ref-n-1)
        a=polyval(p,-n-1);
        ambf(i,3)=ambf(i,3)-round(ambf(i,3)-a);
    end
end

% plot estimated ambiguity -----------------------------------------------------
function plotamb(time,amb,f,minep)
fname='Times New Roman';
fsize=9;

figure('color','w')
axes('position',[0.1,0.1,0.86,0.84],'fontname',fname,'fontsize',fsize)
hold on, box on, grid on

for i=1:size(amb,2)
    j=find(~isnan(amb(:,i)));
    
    ti=time(j);
    ambi=amb(j,i);
    
    % detect phase bias break
    k=find(abs(diff(ti))>300|abs(diff(ambi))>0.5);
    for l=[1,k'+1;k',length(ambi)]
        
        if l(2)-l(1)<minep, continue; end
        
        plot(ti(l(1):l(2))/3600,ambi(l(1):l(2)),'.-','markersize',5);
    end
end
xlabel('Time (H)')
ylabel('DD Ambiguity (cycle)')
xlim(time([1,end])/3600)

title(sprintf('Estimated GLONASS Ambiguity L%d',f))
uifunc('moveax',gca,[1 1 1 1]);

% plot inter channel bias ------------------------------------------------------
function plotbias(ambf,f,unit)
fname='Times New Roman';
fsize=9;
if strcmp(unit,'cycle'), yl=2; else yl=30; end

figure('color','w')
axes('position',[0.1,0.1,0.86,0.84],'fontname',fname,'fontsize',fsize)
hold on, box on, grid on

plot(ambf(:,2),ambf(:,3),'o')

p=polyfit(ambf(:,2),ambf(:,3),1);
x=-8:0.1:8;
plot(x,polyval(p,x),'r');

rms=sqrt(mean((polyval(p,ambf(:,2))-ambf(:,3)).^2));
text(0.98,0.98,sprintf('BIAS/FN=%6.2f %s RMS=%6.2f %s',p(1),unit,rms,unit),...
     'unit','normalized','vertical','top','horizontal','right',...
     'fontname',fname,'fontsize',fsize)

xlabel('Frequency Number')
ylabel(sprintf('Bias (%s)',unit))
xlim([-8,8])
ylim([-yl,yl])
set(gca,'xtick',-8:1:8)

title(sprintf('GLONASS Interchannel Bias L%d',f))
uifunc('moveax',gca,[0 1 0 1]);

fprintf('GLONASS Interchannel Bias L%d\n',f);
for i=1:size(ambf,1)
    fprintf('R%02d: fn=%2d  bias=%10.4f %s (n=%4d)\n',ambf(i,[1:3]),unit,ambf(i,4));
end

% number argument --------------------------------------------------------------
function n=tonum(s), if isstr(s), n=str2num(s); else n=s; end
