function [rs,dts]=ephpos(obs,iobs,nav,inav)
% ephpos.m :
% compute satellite position/clock biases with ephemeris.
%
%            Copyright(c) 2007-2011 by T.Takasu, all rights reserved
%
% version: $Revision: 1.2 $ $Date: 2008/06/03 01:26:51 $
% history: 2008/05/31 1.0 new
%          2011/09/13 1.1 support GLONASS and QZSS

C=299792458;
rs=repmat(nan,3,size(obs,1)); dts=repmat(nan,1,size(obs,1));
for i=1:size(obs,1)
    eph=seleph(iobs(1,1),nav,inav,iobs(i,2));
    if isempty(eph)|isnan(obs(i,4)), continue; end
    [rs(:,i),dts(i)]=eph2pos(iobs(i,1)-obs(i,4)/C,eph,iobs(i,2));
end

% select ephemeris -------------------------------------------------------------
function eph=seleph(t,nav,inav,sat)
eph=[]; i=find(inav==sat); if isempty(i), return, end
[tt,j]=min(abs(t-nav(i,end)));
if 1<=sat&sat<=32
    if tt<=7200&nav(i(j),31)==0, eph=nav(i(j),:); end
elseif 33<=sat&sat<=56
    if tt<=1800&nav(i(j),13)==0, eph=nav(i(j),:); end
elseif 56<=sat&sat<=58,
    if tt<=7200, eph=nav(i(j),:); end
end

% compute satellite position and clock-bias ------------------------------------
function [rs,dts]=eph2pos(t,eph,sat)
if 33<=sat&sat<=56
    [rs,dts]=eph2pos_glo(t,eph);
else
    [rs,dts]=eph2pos_gps(t,eph);
end

% compute satellite position and clock-bias for gps ----------------------------
function [rs,dts]=eph2pos_gps(t,eph)
C=299792458; OMGE=7.2921151467E-5; MU=3.986005E14;
tk=t-eph(end); tk=t-eph(7:9)*[1;tk;tk^2]-eph(end);
A=eph(17)^2; e=eph(15); n=sqrt(MU/A^3)+eph(12); M=eph(13)+n*tk;
E=M; Ek=0; while abs(E-Ek)>1E-12, Ek=E; E=M+e*sin(Ek); end
p=atan2(sqrt(1-e^2)*sin(E),cos(E)-e)+eph(24);
r=[p;A*(1-e*cos(E));eph(22)+eph(26)*tk]+eph([16,14;11,23;21,19])*[sin(2*p);cos(2*p)];
OMG=eph(20)+(eph(25)-OMGE)*tk-OMGE*eph(18);
rs=r(2)*Rz(-OMG)*Rx(-r(3))*[cos(r(1));sin(r(1));0];
dts=eph(7:9)*[1;tk;tk^2]-2*sqrt(MU*A)*e*sin(E)/C^2-eph(32);

% compute satellite position and clock-bias for glonass ------------------------
function [rs,dts]=eph2pos_glo(t,eph)
STEP=30; tk=t-eph(end); tk=t-eph(7:8)*[-1;tk]-eph(end);
x=eph([10,14,18,11,15,19])'*1E3;
while abs(tk)>1e-12
    if tk>STEP, h=STEP; elseif tk<-STEP, h=-STEP; else h=tk; end
    x=integ_glo(x,h,eph([12,16,20])'*1E3);
    tk=tk-h;
end
rs=x(1:3); dts=eph(7:8)*[-1;tk];

% integrate glonass orbit ------------------------------------------------------
function x=integ_glo(x,h,acc)
k1=ode_glo(x       ,acc);
k2=ode_glo(x+k1*h/2,acc);
k3=ode_glo(x+k2*h/2,acc);
k4=ode_glo(x+k3*h  ,acc);
x=x+(k1+2*k2+2*k3+k4)*h/6;

% glonass orbit differential equations -----------------------------------------
function xdot=ode_glo(x,acc)
RE=6378136; MU=3.9860044E14; J2=1.0826257E-3; OMG=7.292115E-5;
r=norm(x(1:3)); A=1.5*J2*MU*RE^2/r^5; B=5*x(3)^2/r^2;
xdot=[x(4:6);-MU/r^3*x(1)-A*x(1)*(1-B)+OMG^2*x(1)+2*OMG*x(5)+acc(1);
             -MU/r^3*x(2)-A*x(2)*(1-B)+OMG^2*x(2)-2*OMG*x(4)+acc(2);
             -MU/r^3*x(3)-A*x(3)*(3-B)+acc(3)];

% coordinate rotation matrix ---------------------------------------------------
function R=Rx(t), R=[1,0,0;0,cos(t),sin(t);0,-sin(t),cos(t)];
function R=Rz(t), R=[cos(t),sin(t),0;-sin(t),cos(t),0;0,0,1];
