function [t0,obs,iobs,nav,inav,ionp,leaps]=readrnx(files)
% readrnx.m :
% read rinex obs data and navigation messages from files.
%
% argout :
%    t0        = date number by datenum.m (days)
%    obs (i,:) = observation data [L1 L2 L5 C1 P2 P5 S1 S2 S5;...] (nan:no data)
%    iobs(i,:) = observation data index [time sat rcv LLI1 LLI2 LLI5;...]
%    nav (i,:) = navigation messages stored by rinex order.
%    inav(i,:) = navigation messages satellite index
%    ionp      = ionospheric parameters [a0,a1,a2,a3;b0,b1,b2,b3]
%    leaps     = leap seconds (s)
%
%            Copyright(c) 2007-2011 by T.Takasu, all rights reserved
%
% version: $Revision: 1.2 $ $Date: 2008/06/03 01:26:51 $
% history: 2008/05/23 1.1 new
%          2009/01/06 1.2 fix bug on error-stop in reading rinex obs data
%          2011/09/13 1.3 support GLONASS, QZSS and L5
%                         add argout leaps
%                         fix bug on reading more than 8 obs types

t0=[]; obs=[]; iobs=[]; nav=[]; inav=[]; ionp=repmat(nan,2,4); leaps=15; rcv=0;
for i=1:length(files)
    f=fopen(files{i},'rt');
    if f<0, disp(['file open error :',files{i}]); continue; end
    disp(['reading rinex file ... : ',files{i}]);
    [vrnx,trnx,tsys,tobs,ion,lps]=readrnxh(f);
    if vrnx>=3
        disp(['rinex 3 is not supported :',files{i}]);
        fclose(f);
        continue;
    end
    if trnx=='O'
        [t0,obsk,iobsk]=readrnxo(t0,f,tobs);
        rcv=rcv+1; iobsk(:,3)=rcv; obs=[obs;obsk]; iobs=[iobs;iobsk];
    elseif trnx=='N'&any(tsys==' G') % GPS NAV
        [v,i]=readrnxn(f); nav=[nav;v]; inav=[inav;i];
        if size(ion,1)>=2, ionp=ion; end
    elseif trnx=='N'&tsys=='J' % QZSS NAV
        [v,i]=readrnxq(f); nav=[nav;v]; inav=[inav;i];
    elseif trnx=='G' % GLONASS NAV
        [v,i]=readrnxg(f); nav=[nav;v]; inav=[inav;i];
    end
    if ~isempty(lps), leaps=lps; end
    fclose(f);
end
if ~isempty(obs)
    [iobs,i]=sortrows(iobs); obs=obs(i,:);
    td=floor(iobs(1,1)/86400); t0=t0+td; iobs(:,1)=iobs(:,1)-86400*td;
end
if ~isempty(nav)
    [n,i]=unique(nav(:,1:33),'rows'); nav=nav(i,:); inav=inav(i,:);
end

% read rinex header -----------------------------------------------------------
function [vrnx,trnx,tsys,tobs,ion,lps]=readrnxh(f)
vrnx=0; trnx=''; tsys=''; tobs={}; ion=[]; lps=[];
while 1
    s=fgetl(f); if ~isstr(s), break, end
    label=subs(s,61,20);
    if findstr(label,'RINEX VERSION / TYPE')
        vrnx=s2n(s,6,3); trnx=subs(s,21,1); tsys=subs(s,41,1);
    elseif findstr(label,'# / TYPES OF OBSERV')
        p=11;
        for i=1:s2n(s,1,6)
            if p>=65, s=fgetl(f); p=11; end
            tobs{i}=subs(s,p,2); p=p+6;
        end
    elseif findstr(label,'ION ALPHA')
        s=strrep(s,'D','E'); s=strrep(s,'d','E');
        for n=1:4, ion(1,n)=s2n(s,n*12-9,12); end
    elseif findstr(label,'ION BETA')
        s=strrep(s,'D','E'); s=strrep(s,'d','E');
        for n=1:4, ion(2,n)=s2n(s,n*12-9,12); end
    elseif findstr(label,'LEAP SECONDS')
        lps=s2n(s,1,6);
    elseif findstr(label,'END OF HEADER'), break, end
end

% read rinex observation data --------------------------------------------------
function [t0,obs,iobs]=readrnxo(t0,f,tobs)
to={'L1','L2','L5','C1','P2','C5','S1','S2','S5'}; ind=zeros(1,length(tobs));
for i=1:length(to), j=find(strcmp(to{i},tobs)); if ~isempty(j), ind(j)=i; end, end
if all(ind~=4), ind(strcmp('P1',tobs))=4; end
if all(ind~=5), ind(strcmp('C2',tobs))=5; end
obs=[]; iobs=[]; n=0;
while 1
    s=fgetl(f); if ~isstr(s), break, end
    e=s2e(s,1,26);
    if length(e)>=6
        if e(1)<80, y=2000; else y=1900; end
        if isempty(t0), t0=datenum(e(1)+y,e(2),e(3)); end
        tt=(datenum(e(1)+y,e(2),e(3))-t0)*86400+e(4:6)*[3600;60;1];
        ns=s2n(s,30,3);
        if size(obs,1)<n+ns
            obs=[obs;zeros(65535,length(to))]; iobs=[iobs;zeros(65535,3+3)];
        end
        iobs(n+(1:ns),1)=tt; p=33;
        for i=1:ns
            if p>=69, s=fgetl(f); p=33; end
            iobs(n+i,2)=satno(subs(s,p,1),s2n(s,p+1,2)); p=p+3;
        end
        for i=1:ns
            s=fgetl(f); p=1;
            for j=1:length(tobs)
                if p>=80, s=fgetl(f); p=1; end
                if ind(j)>0
                    obs(n+i,ind(j))=s2n(s,p,14);
                    if ind(j)<=3
                        ff=s2n(s,p+14,1); if isnan(ff), ff=0; end
                        iobs(n+i,ind(j)+3)=ff;
                    end
                end
                p=p+16;
            end
        end
        n=n+ns;
    end
end
if ~isempty(obs)
    i=find(1<=iobs(:,2)&iobs(:,2)<=32+24+3); % GPS/GLO/QZS
    obs=obs(i,:); iobs=iobs(i,:); obs(obs==0)=nan;
end

% read rinex navigation message ------------------------------------------------
function [nav,inav]=readrnxn(f)
nav=zeros(1024,37); inav=zeros(1024,1); n=0; m=1;
while 1
    s=fgetl(f); if ~isstr(s) break, end
    s=strrep(s,'D','E'); s=strrep(s,'d','E');
    if m==1, prn=s2n(s,1,2); a=s2e(s,3,30); else a=[a,s2n(s,4,19)]; end
    for p=23:19:61, a=[a,s2n(s,p,19)]; end
    if m<8, m=m+1; else n=n+1; nav(n,:)=a; inav(n,1)=satno('G',prn); m=1; end
end
nav=nav(1:n,:); inav=inav(1:n,:); 

% read rinex glonass navigation message ----------------------------------------
function [nav,inav]=readrnxg(f)
nav=zeros(1024,37); inav=zeros(1024,1); n=0; m=1;
while 1
    s=fgetl(f); if ~isstr(s) break, end
    s=strrep(s,'D','E'); s=strrep(s,'d','E');
    if m==1, prn=s2n(s,1,2); a=s2e(s,3,30); else a=[a,s2n(s,4,19)]; end
    for p=23:19:61, a=[a,s2n(s,p,19)]; end
    if m<4, m=m+1; else n=n+1; nav(n,1:21)=a; inav(n,1)=satno('R',prn); m=1; end
end
nav=nav(1:n,:); inav=inav(1:n,:); 

% read rinex qzss navigation message -------------------------------------------
function [nav,inav]=readrnxq(f)
nav=zeros(1024,37); inav=zeros(1024,1); n=0; m=1;
while 1
    s=fgetl(f); if ~isstr(s) break, end
    s=strrep(s,'D','E'); s=strrep(s,'d','E');
    if m==1, prn=s2n(s,2,2); a=s2e(s,4,30); else a=[a,s2n(s,5,19)]; end
    for p=24:19:62, a=[a,s2n(s,p,19)]; end
    if m<8, m=m+1; else n=n+1; nav(n,:)=a; inav(n,1)=satno('J',prn); m=1; end
end
nav=nav(1:n,:); inav=inav(1:n,:); 

% satellite number -------------------------------------------------------------
function sat=satno(sys,prn)
off=[0 0 32 56]; 
i=strfind(' GRJ',sys); if ~isempty(i), sat=prn+off(i); else sat=0; end

% string to number/epoch/substring  --------------------------------------------
function a=s2n(s,p,n), a=sscanf(subs(s,p,n),'%f'); if isempty(a), a=nan; end
function e=s2e(s,p,n), e=sscanf(subs(s,p,n),'%d %d %d %d %d %f',6)';
function s=subs(s,p,n), s=s(p:min(p+n-1,length(s)));
